#cups-install.sh
#v0.1f, pakt, 12sep06
#Changes from v0.1c:
#- verifies any *.ppd file in cups-install dir with 'cupstestppd'
#- added 'network' to choice of printer types
#- starts default browser with CUPS web interface from script
#- checks for missing perl modules only in Puppy 2.01/2.02 (perl 5.8.6).
#  (Puppy 2.10, using perl 5.8.8, has the missing modules)
#- checks that cupsd is running before starting browser
#- backs up original /etc/rc.d/rc.local to /etc/rc.d/rc.local_cups
#- changed to high-capacity download server ftp://slackware.cs.utah.edu
#
#Simplified and improved CUPS install script for Puppy 1.x and 2.x
#combining rarsa's three CUPS installation scripts. The script adds
#the installation of gimp-print-4.2.7-i486-1.tgz which contains
#numerous additional printer drivers.
#
#The following updated CUPS files from Slackware 10.2 are used and must
#be in the same directory as the cups-install.sh script:
#
#- cups-1.1.23-i486-1.tgz
#- espgs-8.15rc4-i486-1.tgz
#- hpijs-1.7-i486-1.tgz
#- gimp-print-4.2.7-i486-1.tgz
#
#as well as these files:
#
#- cupsconf.tar.gz (from rarsa's CUPS installation package)
#- optional *.ppd file for your printer (from www.linuxprinting.org)
#
#If local copies of the CUPS files are not found, they will be
#automatically downloaded from the internet and saved in the same
#directory as the cups-install.sh script.
#
#If during CUPS installation your printer isn't listed, then a *.ppd
#file for your printer will need to be downloaded manually from
#www.linuxprinting.org. Copy the .ppd file to the same directory as the
#cups-install.sh script, then re-run this script and it will be
#installed automatically.
#
#The script will ask if a parallel, USB or network printer is being
#installed, load the appropriate module and add CUPS startup commands
#to /etc/rc.d/rc.local.
#
#Just before the script ends, it starts up the default browser with the
#CUPS web interface in the background and then prints cups-conf.txt,
#giving further instructions for configuring your printer.
#
#If your browser doesn't start automatically, start it with this URL:
#  http://localhost:631/
#Now can add your printer and configure it in the CUPS web interface.

SCRIPTDIR="`pwd`"

#Module names differ between 2.4.x and 2.6.x kernels
#Get kernel version: 2.4.x sets KVER=4, 2.6.x sets KVER=6
KVER=`uname -r | tr ".-" "  " | cut -d" " -f2`

#Sanity checks
#Check if perl is installed in Puppy
perl -v > /dev/null
if [ ! $? -eq 0 ];then
  echo ""
  echo "ERROR: perl is not installed. CUPS will not work without perl."
  echo ""
  echo "The easiest way to install perl is as follows:"
  echo ""
  echo "For Puppy 1.xx get usr_devx.sfs from the Puppy download site"
  echo "and copy it to the same location as your pup001 file. Then"
  echo "re-boot Puppy and run this script again."
  echo ""
  echo "For Puppy 2.xx get devx_2xx.sfs from the Puppy download site"
  echo "and copy it to the same location as your pup_save.3fs file."
  echo "Then re-boot Puppy and run this script again."
  echo ""
  exit 1
fi
PERLVER=`perl -v | grep "This is perl" | cut -f 4 -d " " | sed 's/v//'`
#Check for missing perl modules in Puppy 2.01/2.02
if [ "$PERLVER" = "5.8.6" ]; then
  if [ ! -d /usr/lib/perl5/5.8.6/i486-linux/auto/POSIX ]; then
    echo ""
    echo "ERROR: Puppy 2.01 and 2.02 are missing some perl modules"
    echo "required by CUPS. Please download perl_missing.tar.gz and"
    echo "install it according to the instructions included with this"
    echo "script, then run this script again."
    echo ""
    exit 1
  fi
fi
if [ ! -f cupsconf.tar.gz ] ; then
  echo ""
  echo "ERROR: You must copy cupsconf.tar.gz to the same folder as this"
  echo "script."
  echo ""
  exit 1
fi

#Try to use local copies of cups-1.1.23-i486-1.tgz, espgs-8.15rc4-i486-1.tgz,
#hpijs-1.7-i486-1.tgz and gimp-print-4.2.7-i486-1.tgz. They must be in the
#same folder as this script. If not found, download them from the internet.
#Note: the file-tests bypass wget's 40sec timeout trying to connect to the
#download site when working offline. wget always tries to connect even when
#a local copy of the file is present to confirm that the file is complete.
if [ ! -f cups-1.1.23-i486-1.tgz ] ; then
  wget -c ftp://slackware.cs.utah.edu/pub/slackware/slackware-10.2/slackware/a/cups-1.1.23-i486-1.tgz
  if [ ! $? -eq 0 ];then
    echo ""
    echo "ERROR: File cups-1.1.23-i486-1.tgz is missing."
    echo "Local copy not found and download attempt unsuccessful..."
    echo "...terminating the script."
    echo ""
    exit 1
  fi
fi
if [ ! -f espgs-8.15rc4-i486-1.tgz ] ; then
  wget -c ftp://slackware.cs.utah.edu/pub/slackware/slackware-10.2/slackware/ap/espgs-8.15rc4-i486-1.tgz
  if [ ! $? -eq 0 ];then
    echo ""
    echo "ERROR: File espgs-8.15rc4-i486-1.tgz is missing."
    echo "Local copy not found and download attempt unsuccessful..."
    echo "...terminating the script."
    echo ""
    exit 1
  fi
fi
if [ ! -f hpijs-1.7-i486-1.tgz ] ; then
  wget -c ftp://slackware.cs.utah.edu/pub/slackware/slackware-10.2/slackware/ap/hpijs-1.7-i486-1.tgz
  if [ ! $? -eq 0 ];then
    echo ""
    echo "ERROR: File hpijs-1.7-i486-1.tgz is missing."
    echo "Local copy not found and download attempt unsuccessful..."
    echo "...terminating the script."
    echo ""
    exit 1
  fi
fi
if [ ! -f gimp-print-4.2.7-i486-1.tgz ] ; then
  wget -c ftp://slackware.cs.utah.edu/pub/slackware/slackware-10.2/slackware/ap/gimp-print-4.2.7-i486-1.tgz
  if [ ! $? -eq 0 ];then
    echo ""
    echo "ERROR: File gimp-print-4.2.7-i486-1.tgz is missing."
    echo "Local copy not found and download attempt unsuccessful..."
    echo "...terminating the script."
    echo ""
    exit 1
  fi
fi

#Ask if installing a parallel, USB or network printer
#Parallel sets PNUM=1, USB sets PNUM=2, Network sets PNUM=3
#For network printer - add only command to start CUPS to rc.local. See
#further down in script.
PNUM="0"
echo ""
while [ $PNUM != "1" ] && [ $PNUM != "2" ] && [ $PNUM != "3" ];do
  echo -n "Printer to install:  1-Parallel, 2-USB, 3-Network (1,2,3): "
  read PNUM
done
echo ""

if [ ! -f /etc/rc.d/rc.local_cups ] ; then
  #Backup original rc.local file
  cp -afv /etc/rc.d/rc.local /etc/rc.d/rc.local_cups
else
  #Use clean backup copy for CUPS modifications
  cp -afv /etc/rc.d/rc.local_cups /etc/rc.d/rc.local
fi

#Install port driver for correct kernel and printer type
#Modprobe info garnered from the "CUPS Setup Wiki"
case $KVER in
 4)#Using 2.4.x kernel
  if [ $PNUM -eq 1 ] || [ $PNUM -eq 2 ];then
    #For parallel port and usb printers (they use the same module)
    modprobe printer
    #Configure rc.local to load the port driver at startup
    grep "modprobe printer" /etc/rc.d/rc.local >/dev/null
    if [ ! $? = 0 ] ; then
      #Add to rc.local, but first make sure we start on a new line
      echo "" >> /etc/rc.d/rc.local
      echo "# Load the printer module" >> /etc/rc.d/rc.local
      echo "modprobe printer" >> /etc/rc.d/rc.local
    fi
  fi
  ;;
 6)#Using 2.6.x kernel
  if [ $PNUM -eq 1 ];then
    #For parallel port printers
    #NOTE: Puppy 2.xx loads this module automatically
    echo "Module lp is already loaded..."
    echo ""
    #modprobe lp
    #Configure rc.local to load the port driver at startup
    #grep "modprobe lp" /etc/rc.d/rc.local >/dev/null
    #if [ ! $? = 0 ] ; then
      #Add to rc.local, but first make sure we start on a new line
      #echo "" >> /etc/rc.d/rc.local
      #echo "# Load the parallel port printer module" >> /etc/rc.d/rc.local
      #echo "modprobe lp" >> /etc/rc.d/rc.local
    #fi
  fi
  if [ $PNUM -eq 2 ];then
    #For usb printers
    modprobe usblp
    #Configure rc.local to load the port driver at startup
    grep "modprobe usblp" /etc/rc.d/rc.local >/dev/null
    if [ ! $? = 0 ] ; then
      #Add to rc.local, but first make sure we start on a new line
      echo "" >> /etc/rc.d/rc.local
      echo "# Load the usb printer module" >> /etc/rc.d/rc.local
      echo "modprobe usblp" >> /etc/rc.d/rc.local
    fi
  fi
  ;;
esac

#Install CUPS files
mkdir /tmp/cups
mkdir /tmp/espgs
mkdir /tmp/hpijs
mkdir /tmp/gimp
cd /tmp/cups
tar -xzf ${SCRIPTDIR}/cups-1.1.23-i486-1.tgz
sh install/doinst.sh
cp -afv etc/* /etc/
cp -afv usr/* /usr/
cp -afv var/* /var/

#Install sample configuration files
cd /etc/cups
tar -xzf ${SCRIPTDIR}/cupsconf.tar.gz
#Set proper permissions
chmod -R 666 *
chown root /usr/bin/lppasswd

#Install ESP Ghostscript
cd /tmp/espgs
tar -xzf ${SCRIPTDIR}/espgs-8.15rc4-i486-1.tgz
sh install/doinst.sh
cp -afv etc/* /etc/
cp -afv usr/* /usr/

#Install hpijs (Hewlett-Packard inkjet drivers for Ghostscript)
#The IJS server is apparently needed even for non-hp printers
cd /tmp/hpijs
tar -xzf ${SCRIPTDIR}/hpijs-1.7-i486-1.tgz
sh install/doinst.sh
cp -afv usr/* /usr/

cd /tmp/gimp
tar -xzf ${SCRIPTDIR}/gimp-print-4.2.7-i486-1.tgz
sh install/doinst.sh
cp -afv etc/* /etc/
cp -afv usr/* /usr/

# If a .ppd file is present, test it. If valid, install it or exit script
if [ -f ${SCRIPTDIR}/*.ppd ] ; then
  echo ""
  echo "*** Testing for valid CUPS .ppd file:"
  echo ""
  cupstestppd ${SCRIPTDIR}/*.ppd
  if [ ! $? = 0 ] ; then
    echo ""
    echo "ERROR in .ppd file - this file cannot be used by CUPS. Please"
    echo "replace it with a valid CUPS .ppd file and re-start this script."
    echo ""
    exit 1
  fi
  echo ""
  sleep 3
  cp -afv ${SCRIPTDIR}/*.ppd /usr/share/cups/model/
fi

#Configure rc.local to start cups at startup
chmod +x /etc/rc.d/rc.cups
grep "/etc/rc.d/rc.cups start" /etc/rc.d/rc.local >/dev/null
if [ ! $? = 0 ] ; then
  #Add to rc.local, but first make sure we start on a new line
  echo "" >> /etc/rc.d/rc.local
  echo "# Start cups" >> /etc/rc.d/rc.local
  echo "/etc/rc.d/rc.cups start" >> /etc/rc.d/rc.local
fi

#Start cups
/etc/rc.d/rc.cups start
#Make sure cups daemon is running
sleep 1
ps | grep cupsd | grep /usr/sbin/cupsd >/dev/null
if [ ! $? = 0 ] ; then
  #cupsd is not running, try to start it again
  /etc/rc.d/rc.cups start
  sleep 1
  ps | grep cupsd | grep /usr/sbin/cupsd >/dev/null
  if [ ! $? = 0 ] ; then
    echo ""
    echo "ERROR: Cannot start CUPS daemon 'cupsd'."
    echo ""
    exit 1
  fi
fi

#Start browser with CUPS web interface
defaultbrowser http://localhost:631 &

#Print configuration instructions using the CUPS web interface
cat ${SCRIPTDIR}/cups-conf.txt | more

cd ${SCRIPTDIR}

###END###